/*************************************************** */
/* Rule Set Based Access Control                     */
/*                                                   */
/* Author and (c) 1999-2005: Amon Ott <ao@rsbac.org> */
/*                                                   */
/* Last modified: 31/May/2005                        */
/*************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <linux/kdev_t.h>
#include <termios.h>
#include <sys/resource.h>
#include <rsbac/types.h>
#include <rsbac/rc_types.h>
#include <rsbac/syscalls.h>
#include <rsbac/getname.h>
#include <rsbac/error.h>
#include "nls.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

char * progname;
struct termios old_term;
struct termios tmp_term;

void use(void)
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
      printf(gettext("Use: %s [-v] new_role_id prog args\n"), progname);
      printf(gettext("This program will set the process rc_role to new_role and then\n"));
      printf(gettext("execute prog via execvp()\n"));
      printf(gettext("  -P = ask for password\n"));
      printf(gettext("-v = verbose\n"));
    }

int env_init(void)
{
	int res;
	
	struct rlimit rlimit;

	rlimit.rlim_cur = 0;
	rlimit.rlim_max = 0;
	res = setrlimit(RLIMIT_CORE, &rlimit);
	if (res)
		fprintf(stderr, gettext("Warning: unable to set resource limits.\n"));	
	
	return res;
}

int main(int argc, char ** argv)
{
  int res = 0;
  int verbose = 0;
  int ask_pass = 0;
  char * pass;
  rsbac_rc_role_id_t role;

  env_init();
  pass = malloc(RSBAC_MAXNAMELEN);
  if (!pass)
	  exit(-ENOMEM);
  res = mlock(pass, RSBAC_MAXNAMELEN);
  if(res)
	  fprintf(stderr, gettext("Warning: password was not locked into physical memory.\n"));	
  
  locale_init();

  progname = argv[0];
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'h':
                use();
                return 0;
              case 'v':
                verbose++;
                break;
	      case 'P':
		ask_pass = 1;
		break;
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
                exit(1);
            }
          pos++;
        }
      argv++;
      argc--;
    }
  if (argc >= 3)
    {
      if (ask_pass) {
	      printf("Password: ");
	      if (isatty(STDIN_FILENO)) {
		      tcgetattr(STDIN_FILENO, &old_term);
		      memcpy(&tmp_term, &old_term, sizeof(old_term));
		      tmp_term.c_lflag &= ~(ECHO);
		      tcsetattr(STDIN_FILENO, TCSAFLUSH, &tmp_term);
	      }
	      res = scanf("%254s", pass);
	      if (isatty(STDIN_FILENO))
		      tcsetattr(STDIN_FILENO, TCSAFLUSH, &old_term);
	      printf("\n");
	      if (!res) {
		      fprintf(stderr, gettext("%s: invalid password!\n"), progname);
		      exit(1);
	      }
      }
	    
      role=strtol(argv[1],0,10);
      if(verbose)
        printf(gettext("%s: executing %s with role %i\n"),
               progname,argv[2],role);
      if (pass)
	      res = rsbac_rc_change_role(role, pass);
      else
	      res = rsbac_rc_change_role(role, 0);
      error_exit(res);
      res = execvp(argv[2],&argv[2]);
      error_exit(res);
    }
  else
    {
      use();
      return 1;
    }
  return (res);
}

